"""
🚀 Methods that are used in the API, to make calls to the API.

Includes:

- errors: Exceptions that are used in the API.
- methods: Methods that are used in the API.

"""

__all__ = ["errors", "methods"]

from . import errors
from . import methods
